// use example DS1307RTC SetTime to set RTC time to PC
// 4, 7 segment display shows time
// LCD 20x4, DS3231 RTC and BMP  uses I2c connections
// DHT11 measures inside temp and hum (less acurate)
// DHT22 measures outside temp and hum and calculates dew point
// added milli instead of delay to rotate through screens
// version 4.1 09/11/18

#include <RTClib.h>
#include <Wire.h>
#include <LiquidCrystal_I2C.h>
#include <DS1307RTC.h>
#include <SPI.h>
#include <Adafruit_Sensor.h>
#include <Adafruit_BMP280.h>
#include <DHT.h>
#include <dht11.h>
#include <Time.h>
#include <TimeLib.h>
#include <TM1637.h>

#define DHT1PIN 7        // DHT11 TO PIN 7
#define DHT2PIN 13       // DHT22 to PIN 13
#define DHT2TYPE DHT22   // DHT22  
#define DHT1TYPE DHT11   // DHT11
#define CLK 9            // 7 segment display PIN CLK
#define DIO 8            // 7 segment display PIN DIO

TM1637 Display1(CLK, DIO);
int8_t Digits[] = {0, 1, 2, 3};
int hours;
int minutes;
boolean alterna;

DHT dht2 (DHT2PIN, DHT2TYPE);   //outside temp,hum
DHT dht1 (DHT1PIN, DHT1TYPE);   //inside temp,hum
Adafruit_BMP280 bme;

LiquidCrystal_I2C lcd(0x27, 20, 4); // Set the LCD I2C address

int chk;

float temp1;      // inside
float hum1;       // inside
float temp2;      // outside
float hum2;       // outside
float Pressure;
float dp;         // Dew Point Outside
float temp2f; //temp outside in F
float hif;    //Heat index in F
float hic;    //Heat Index in C

unsigned long switchMillis = 0;
unsigned long interval = 5000;  // 5 seconds for each lcd screen
byte count = 0;

boolean firstScreenFlag = false;
boolean secondScreenFlag = false;
boolean thirdScreenFlag = false;
boolean fouthScreenFlag = false;

void setup()
{

  Serial.begin(9600);

  Display1.set();
  Display1.init();

  lcd.init();
  lcd.clear();
  lcd.backlight();
  dht1.begin();
  dht2.begin();
  Wire.begin();

  lcd.setCursor(0, 0);
  lcd.print(F("  Weather Station"));
  lcd.setCursor(3, 2);
  lcd.print(F("Version : 4.1"));
  delay(3000);
  lcd.clear();

  if (!bme.begin())
  {
    while (1);
  }

  lcd.print(F("Real Time Clock"));
  delay(1000);
  lcd.setCursor(0, 1);
  lcd.print(F("DHT11 Inside Temp"));
  delay(1000);
  lcd.setCursor(0, 2);
  lcd.print(F("DHT22 Outside Temp"));
  delay(1000);
  lcd.setCursor(0, 3);
  lcd.print(F("BMP280 Pressure"));
  delay(3000);
  lcd.clear();

}

void loop()
{

  tmElements_t tm;

  if (RTC.read(tm)) {

    

    print2digits(tm.Hour);
    print2digits(tm.Minute);

    hours = tm.Hour;
    minutes = tm.Minute;
    CalculaDigits(hours, minutes);
    if (alterna)
    {
      Display1.point(POINT_OFF);
      alterna = false;
    }
    else
    {
      Display1.point(POINT_ON);
      alterna = true;
    }

  } else {
    if (RTC.chipPresent()) {

    } else {

    }
    delay(9000);
  }
  delay(1000);

  if (millis() - switchMillis > interval)
  {
    count++;

    if (count == 1)
    {
      firstScreenFlag = true;
      count = 1;
    }
    if (count == 2)
    {
      secondScreenFlag = true;
      count = 2;
    }
    if (count == 3)
    {
      thirdScreenFlag = true;
      count = 3;
    }
    if (count == 4)
    {
      fouthScreenFlag = true;
      count = 0;
    }

    switchMillis += interval; //millis();
  }


  if (firstScreenFlag)
  {
    printTime(7, 1, tm);
    firstScreenFlag = false;
  }

  if (secondScreenFlag)
  {
    printTemp();
    secondScreenFlag = false;
  }

  if (thirdScreenFlag)
  {
    printIndex();
    thirdScreenFlag = false;
  }

  if (fouthScreenFlag)
  {
    printPressure();
    fouthScreenFlag = false;
  }


  


}

void printTime(int character, int line, tmElements_t tm)
{

  lcd.clear();
  String seconds, minutes;
  lcd.setCursor(character, line);
  lcd.print(tm.Hour);
  lcd.print(F(":"));
  if (tm.Minute < 10)
  {
    minutes = "0" + String(tm.Minute);
    lcd.print(minutes);
  } else
  {
    lcd.print(tm.Minute);
  }

  lcd.setCursor(5, 2);
  lcd.print(tm.Day);
  lcd.print(F("/"));
  lcd.print(tm.Month);
  lcd.print(F("/"));
  lcd.print(tmYearToCalendar(tm.Year));
}

void printTemp()
{
  hum1 = dht1.readHumidity();               //inside
  temp1 = dht1.readTemperature();           //inside
  hum2 = dht2.readHumidity();               //outside
  temp2 = dht2.readTemperature();           //outside
    
  lcd.clear();
  lcd.setCursor(0, 0);
  lcd.print(F("   Inside    Outside"));
  lcd.setCursor(0, 2);
  lcd.print(F("T: "));
  lcd.print(temp1, 1);
  lcd.print(F(" "));
  lcd.print((char)223);
  lcd.print(F("C"));
  lcd.setCursor(13, 2);
  lcd.print(temp2, 1);
  lcd.print(F(" "));
  lcd.print(( char)223);
  lcd.print(F("C"));

  lcd.setCursor(0, 3);
  lcd.print(F("H:  "));
  lcd.print(hum1, 0);
  lcd.print(F(" %"));
  lcd.setCursor(14, 3);
  lcd.print(hum2, 0);
  lcd.print(F(" %"));

  

}
void printIndex()
{
  
  hum2 = dht2.readHumidity();               //outside
  temp2 = dht2.readTemperature();           //outside
  temp2f = dht2.readTemperature(true);
  
  double Point = log(hum2 / 100) + ((17.62 * temp2) / (243.5 + temp2));
  double dp = 243.5 * Point / (17.62 - Point);

  lcd.clear();
  lcd.setCursor(0,0);
  lcd.print(F("  Outside Readings"));
  lcd.setCursor(0, 2);
  lcd.print(F("Dew Point : "));
  lcd.print(dp, 1);
  lcd.print(F(" "));
  lcd.print((char)223);
  lcd.print(F("C"));

  hif = dht2.computeHeatIndex(temp2f, hum2);
  hic = dht2.computeHeatIndex(temp2, hum2, false);
  
  lcd.setCursor(0, 3);
  lcd.print(F("Heat Index: "));
  lcd.print(hic, 1);
  lcd.print(F(" "));
  lcd.print((char)223);
  lcd.print(F("C"));
  
}
void printPressure()
{
  Pressure = (bme.readPressure() / 100);    // 100 Pa = 1 millibar
  
  lcd.clear();
  lcd.setCursor(0, 1);
  lcd.print(F("  Press: "));
  lcd.print(Pressure);
  lcd.print(F(" mb"));
  lcd.setCursor(0, 3);
  lcd.print(F("Weather :"));

  if (Pressure <= 970); {
    lcd.setCursor(13, 3);
    lcd.print(F("         "));
    lcd.setCursor(13, 3);
    lcd.print(F("Stormy"));
  }
  if (Pressure > 970) {
    lcd.setCursor(11, 3);
    lcd.print(F("         "));
    lcd.setCursor(11, 3);
    lcd.print(F("Much Rain"));
  }
  if (Pressure > 996) {
    lcd.setCursor(11, 3);
    lcd.print(F("         "));
    lcd.setCursor(13, 3);
    lcd.print(F("Rain"));
  }
  if  (Pressure > 1006) {
    lcd.setCursor(11, 3);
    lcd.print(F("         "));
    lcd.setCursor(11, 3);
    lcd.print(F("Variable"));
  }
  if  (Pressure > 1030) {
    lcd.setCursor(11, 3);
    lcd.print(F("         "));
    lcd.setCursor(13, 3);
    lcd.print(F("Fair"));
  }
  if (Pressure > 1040) {
    lcd.setCursor(11, 3);
    lcd.print(F("         "));
    lcd.setCursor(11, 3);
    lcd.print(F("Set Fair"));
  }
  if (Pressure > 1060) {
    lcd.setCursor(11, 3);
    lcd.print(F("         "));
    lcd.setCursor(11, 3);
    lcd.print(F("Very Dry"));
  }
}

void print2digits(int number) {
  if (number >= 0 && number < 10) {
    Serial.write('0');
  }
  Serial.print(number);
}

void CalculaDigits( int hour, int min)
{
  int8_t Digit0 = min % 10 ;
  int8_t Digit1 = (min % 100) / 10 ;
  int8_t Digit2 = hour % 10 ;
  int8_t Digit3 = (hour % 100) / 10 ;

  Digits[3] = Digit0 ;
  Digits[2] = Digit1 ;
  Digits[1] = Digit2 ;
  Digits[0] = Digit3 ;

  Display1.display(Digits);
}